require 'rails/generators'
require 'fileutils'
require_relative '../../shoestrap/shell'
require_relative 'base_generator'

module Shoestrap
  class CmsGenerator < BaseGenerator

    def set_up_shoestrap_cms
      unless shoestrap_cms_already_installed?
        setup_base_controller
        setup_base_views
        setup_navigation
        setup_translation_file
      end
    end

    def add_route
      generate 'resource_route', resource_route_name
    end

    def add_navigation_link
      if kuhsaft_is_installed?
        inject_into_file 'config/cms_navigation.rb', :before => 'primary.dom_class = \'nav\'' do
          "primary.item :#{model_name}_nav, t('cms.#{plural_name}.navigation_title'), #{route_name}_path\n    "
        end
      end
    end

    def add_model_and_migration
      generate 'model', model_name, ARGV[1..-1].join(' ')
      inject_into_file "app/models/#{model_name}.rb", before: 'class' do
        "require 'shoestrap/cms_model'\n\n"
      end

      inject_into_file "app/models/#{model_name}.rb", before: 'end' do
        <<-eos.gsub(/^ {8}/, '').chomp
          include Shoestrap::CMSModel

          # TODO: Define what attributes are shown in the form and permitted by strong parameters
          editable_attributes #{attribute_keys_as_string}

          # TODO: Define what attributes are shown in the index view
          index_attributes #{attribute_keys_as_string}

         eos
      end
    end

    def add_resource_translations
      template 'translations/resource.yml.erb', "config/locales/de/#{model_name}.yml"
    end

    def add_controller
      generate 'controller', controller_name, " --controller-specs=no --view-specs=no"
      gsub_file "app/controllers/#{controller_name}_controller.rb", 'ApplicationController', 'Shoestrap::BaseController'
    end

    private

    def shoestrap_cms_already_installed?
      File.exists?('app/controllers/shoestrap/base_controller.rb') && File.exists?('config/cms_navigation.rb')
    end

    def setup_base_controller
      empty_directory 'app/controllers/shoestrap'
      if kuhsaft_is_installed?
        template 'base_controller_with_kuhsaft.rb', 'app/controllers/shoestrap/base_controller.rb'
      else
        template 'base_controller_without_kuhsaft.rb', 'app/controllers/shoestrap/base_controller.rb'
      end
    end

    def setup_base_views
      directory 'inherited_views', 'app/views/shoestrap'
    end

    def setup_navigation
      if kuhsaft_is_installed?
        nav_dir = 'app/views/kuhsaft/cms/admin'
        empty_directory nav_dir
        copy_file '_main_navigation.html.haml', "#{nav_dir}/_main_navigation.html.haml"
        copy_file 'cms_navigation.rb', "config/cms_navigation.rb"
      end
    end

    def setup_translation_file
      empty_directory 'config/locales/de'
      copy_file 'translations/shoestrap_base.yml', 'config/locales/de/shoestrap_base.yml'
    end

    def resource_route_name
      "cms/#{model_name}"
    end

    def model_name
      ARGV.first.downcase
    end

    def plural_name
      model_name.pluralize
    end

    def controller_name
      "cms/#{plural_name}"
    end

    def route_name
      controller_name.gsub('/', '_')
    end

    def attributes
      ARGV[1..-1].map {|x| x.split(':').first }
    end

    def attribute_keys_as_string
      ARGV[1..-1].map {|x| ":#{x.split(':').first}" }.join(', ')
    end
  end
end
