require 'active_support/inflector'
require_relative 'shell'
require_relative '../generators/shoestrap/base_generator'
Dir[File.join(File.dirname(__FILE__), '../generators/shoestrap/*_generator.rb')].each{ |f| require f }

module Shoestrap
  module ApplicationGenerator
    extend self

    APPLICATION_TEMPLATE = File.join(File.dirname(__FILE__), '../../templates/application_generator/application_template.rb')

    def start(application_name, options)
      puts "[shoestrap] Generating new Rails Application #{application_name}"
      rails_new_with_template(application_name, options[:database])
      shoestrap_generators(application_name, options)
    end

    def shoestrap_generators(application_name, options)
      Dir.chdir(application_name) do
        # Sorry guys, a tiny little bit of setup that is very SC specific is added
        # by an additional gem we maintain privately.
        # This ugly hack allows us to run it here while not messing
        # with your setup.
        #
        if ENV['GIVE_ME_SPECIAL_SAUCE']
          apply_special_sauce(application_name)
        end

        install_gems(required_gems(options))
        generators = [
          'database', 'smacss_files', 'coffee_files', options[:css],
          'bdd', 'mailcatcher', 'simple_navigation', 'view_files'
        ]
        generators << 'kuhsaft' if options[:kuhsaft]
        generators.each{|name| call_generator name }
      end
    end

    def install_gems(gemlist)
      Shoestrap::GemInstallGenerator.start(gemlist)
      Shell.exec('bundle')
      reload_gemfile_for_application
    end

    def call_generator(name)
      "Shoestrap::#{"#{name}_generator".camelcase}".constantize.start
    end

    def required_gems(options)
      Array.new.tap do |gems|
        gems << 'kuhsaft' if options[:kuhsaft]
        gems << 'bootstrap-sass' if options[:css] == 'bootstrap'
        gems << 'zurb-foundation' if options[:css] == 'foundation'
      end
    end

    def apply_special_sauce(application_name)
      Shell.exec "sauce apply . --appname=#{application_name.split('/').last}"
    end

    private
    def rails_new_with_template(application_name, database)
      Shell.exec "rails new #{application_name} -m #{APPLICATION_TEMPLATE} -d #{database}"
    end

    def reload_gemfile_for_application
      ENV['BUNDLE_GEMFILE'] = File.join(Dir.pwd, 'Gemfile')
      require 'bundler'
      Bundler.require(:default)
    end
  end
end
