require File.expand_path('../../../lib/shoestrap/version', __FILE__)

#
# SC Rails Application Template
#
def source_paths
  [File.expand_path(File.dirname(__FILE__))]
end

#
# Locale and Timezone
#
application "config.time_zone = 'Bern'"
application "config.i18n.load_path += Dir[Rails.root.join('config', 'locales', '**', '*.{rb,yml}').to_s]"
application "config.i18n.available_locales = [:de]"
application "config.i18n.default_locale = :de"

#
# Miscellaneous
#
application "config.typekit_key = false"

inject_into_file 'config/application.rb', :after => "Application < Rails::Application" do
  <<-eos
    # Customize generators
    config.generators do |g|
      g.stylesheets false
      g.template_engine :haml
      g.test_framework :rspec
      g.javascripts false
      g.assets false
      g.helper false
    end
  eos
end

#
# Default Gems (others are added to gemfile by individual generators called after app generation)
#
gem 'therubyracer', platforms: :ruby
gem 'simple_form', '3.0.0'
gem 'haml'
gem 'haml-rails'
gem 'simple-navigation'
gem 'mail'
gem 'fabrication'
gem 'inherited_resources'
gem 'rails-i18n'
gem 'i18n-missing_translations'

if ENV['SHOESTRAP_PATH']
  gem 'shoestrap', path: ENV['SHOESTRAP_PATH']
else
  gem 'shoestrap', Shoestrap::VERSION
end

gem 'selectivizr-rails'
gem 'modernizr-rails'
gem 'bourbon'

gem_group :development, :test do
  gem 'pry-rails'
  gem 'terminal-notifier-guard'
  gem 'rspec-rails'
  gem 'guard-rspec'
  gem 'guard-spring'
  gem 'mailcatcher'
  gem 'capybara'
  gem 'capybara-webkit'
  gem 'launchy'
end

gem_group :test do
  gem 'database_cleaner'
end

#
# Application layout
#
remove_file "app/views/layouts/application.html.erb"
copy_file "application.html.haml", "app/views/layouts/application.html.haml"
copy_file '_typekit.html.haml', 'app/views/layouts/_typekit.html.haml' 

#
# Application helper
#
copy_file "application_helper.rb", "app/helpers/application_helper.rb", force: true

#
# Favicons
#
directory "favicons", "app/assets/images/favicons/"
